@@ -43,7 +43,7 @@ from arroyo.errors import (
     OffsetOutOfRange,
     TransportError,
 )
-from arroyo.types import Message, Partition, Position, Topic
+from arroyo.types import BrokerValue, Partition, Topic
 from arroyo.utils.concurrent import execute
 from arroyo.utils.retries import NoRetryPolicy, RetryPolicy
 
@@ -119,12 +119,6 @@ class KafkaConsumer(Consumer[KafkaPayload]):
     prevent uncommitted messages from being consumed multiple times,
     ``commit`` should be called in the partition revocation callback.
 
-    If the `cooperative-sticky` strategy is used, this won't happen as
-    only the incremental partitions are passed to the callback during
-    rebalancing, and any previously assigned partitions will continue
-    from their previous position without being reset to the last committed
-    position.
-
     The behavior of ``auto.offset.reset`` also differs slightly from the
     Confluent consumer as well: offsets are only reset during initial
     assignment or subsequent rebalancing operations. Any other circumstances
@@ -202,10 +196,6 @@ class KafkaConsumer(Consumer[KafkaPayload]):
                 "invalid value for 'enable.auto.offset.store' configuration"
             )
 
-        self.__incremental_cooperative = (
-            configuration.get("partition.assignment.strategy") == "cooperative-sticky"
-        )
-
         # NOTE: Offsets are explicitly managed as part of the assignment
         # callback, so preemptively resetting offsets is not enabled when
         # strict_offset_reset is enabled.
@@ -217,7 +207,7 @@ class KafkaConsumer(Consumer[KafkaPayload]):
         )
 
         self.__offsets: MutableMapping[Partition, int] = {}
-        self.__staged_offsets: MutableMapping[Partition, Position] = {}
+        self.__staged_offsets: MutableMapping[Partition, int] = {}
         self.__paused: Set[Partition] = set()
 
         self.__commit_retry_policy = commit_retry_policy
@@ -274,65 +264,32 @@ class KafkaConsumer(Consumer[KafkaPayload]):
         ) -> None:
             self.__state = KafkaConsumerState.ASSIGNING
 
-            if self.__incremental_cooperative is True:
-                try:
-                    incremental_assignment: MutableSequence[
-                        ConfluentTopicPartition
-                    ] = []
+            try:
+                assignment: MutableSequence[ConfluentTopicPartition] = []
 
-                    for partition in partitions:
-                        if partition.offset >= 0:
-                            incremental_assignment.append(partition)
-                        elif partition.offset == OFFSET_INVALID:
-                            incremental_assignment.append(
-                                self.__resolve_partition_starting_offset(partition)
-                            )
-                        else:
-                            raise ValueError("received unexpected offset")
+                for partition in self.__consumer.committed(partitions):
+                    if partition.offset >= 0:
+                        assignment.append(partition)
+                    elif partition.offset == OFFSET_INVALID:
+                        assignment.append(
+                            self.__resolve_partition_starting_offset(partition)
+                        )
+                    else:
+                        raise ValueError("received unexpected offset")
 
-                    offsets = {
-                        Partition(Topic(i.topic), i.partition): i.offset
-                        for i in incremental_assignment
-                    }
+                offsets = {
+                    Partition(Topic(i.topic), i.partition): i.offset for i in assignment
+                }
 
-                    self.__incremental_assign(offsets)
+                self.__assign(offsets)
 
-                    # Ensure that all partitions are resumed on assignment to avoid
-                    # carrying over state from a previous assignment.
-                    self.resume([p for p in offsets])
+                # Ensure that all partitions are resumed on assignment to avoid
+                # carrying over state from a previous assignment.
+                self.resume([p for p in offsets])
 
-                except Exception:
-                    self.__state = KafkaConsumerState.ERROR
-                    raise
-
-            else:
-                try:
-                    assignment: MutableSequence[ConfluentTopicPartition] = []
-
-                    for partition in self.__consumer.committed(partitions):
-                        if partition.offset >= 0:
-                            assignment.append(partition)
-                        elif partition.offset == OFFSET_INVALID:
-                            assignment.append(
-                                self.__resolve_partition_starting_offset(partition)
-                            )
-                        else:
-                            raise ValueError("received unexpected offset")
-
-                    offsets = {
-                        Partition(Topic(i.topic), i.partition): i.offset
-                        for i in assignment
-                    }
-
-                    self.__assign(offsets)
-
-                    # Ensure that all partitions are resumed on assignment to avoid
-                    # carrying over state from a previous assignment.
-                    self.resume([p for p in offsets])
-
-                except Exception:
-                    self.__state = KafkaConsumerState.ERROR
-                    raise
+            except Exception:
+                self.__state = KafkaConsumerState.ERROR
+                raise
 
             try:
                 if on_assign is not None:
@@ -394,7 +351,9 @@ class KafkaConsumer(Consumer[KafkaPayload]):
 
         self.__consumer.unsubscribe()
 
-    def poll(self, timeout: Optional[float] = None) -> Optional[Message[KafkaPayload]]:
+    def poll(
+        self, timeout: Optional[float] = None
+    ) -> Optional[BrokerValue[KafkaPayload]]:
         """
         Return the next message available to be consumed, if one is
         available. If no message is available, this method will block up to
@@ -450,20 +409,19 @@ class KafkaConsumer(Consumer[KafkaPayload]):
                 raise ConsumerError(str(error))
 
         headers: Optional[Headers] = message.headers()
-        result = Message(
-            Partition(Topic(message.topic()), message.partition()),
-            message.offset(),
+        broker_value = BrokerValue(
             KafkaPayload(
                 message.key(),
                 message.value(),
                 headers if headers is not None else [],
             ),
+            Partition(Topic(message.topic()), message.partition()),
+            message.offset(),
             datetime.utcfromtimestamp(message.timestamp()[1] / 1000.0),
         )
+        self.__offsets[broker_value.partition] = broker_value.next_offset
 
-        self.__offsets[result.partition] = result.next_offset
-
-        return result
+        return broker_value
 
     def tell(self) -> Mapping[Partition, int]:
         """
@@ -494,16 +452,6 @@ class KafkaConsumer(Consumer[KafkaPayload]):
         )
         self.__offsets.update(offsets)
 
-    def __incremental_assign(self, offsets: Mapping[Partition, int]) -> None:
-        self.__validate_offsets(offsets)
-        self.__consumer.incremental_assign(
-            [
-                ConfluentTopicPartition(partition.topic.name, partition.index, offset)
-                for partition, offset in offsets.items()
-            ]
-        )
-        self.__offsets.update(offsets)
-
     def seek(self, offsets: Mapping[Partition, int]) -> None:
         """
         Change the read offsets for the provided partitions.
@@ -583,35 +531,38 @@ class KafkaConsumer(Consumer[KafkaPayload]):
 
         return [*self.__paused]
 
-    def stage_positions(self, positions: Mapping[Partition, Position]) -> None:
-        if self.__state in {KafkaConsumerState.CLOSED, KafkaConsumerState.ERROR}:
-            raise InvalidState(self.__state)
-
-        if positions.keys() - self.__offsets.keys():
-            raise ConsumerError("cannot stage offsets for unassigned partitions")
-
-        self.__validate_offsets(
-            {partition: position.offset for (partition, position) in positions.items()}
-        )
-
+    def stage_offsets(self, offsets: Mapping[Partition, int]) -> None:
+        # This method is on an extremely hot path since it is called
+        # unconditionally before any commit policies are evaluated. Therefore
+        # all of the validation happens in commit_positions.
+        #
+        # We considered moving the concept of "staged offsets" into the
+        # producer, but it turns out that the consumer needs to sometimes clear
+        # staged offsets.
+        #
         # TODO: Maybe log a warning if these offsets exceed the current
         # offsets, since that's probably a side effect of an incorrect usage
         # pattern?
-        self.__staged_offsets.update(positions)
+        self.__staged_offsets.update(offsets)
 
-    def __commit(self) -> Mapping[Partition, Position]:
+    def __commit(self) -> Mapping[Partition, int]:
         if self.__state in {KafkaConsumerState.CLOSED, KafkaConsumerState.ERROR}:
             raise InvalidState(self.__state)
 
+        if self.__staged_offsets.keys() - self.__offsets.keys():
+            raise ConsumerError("cannot stage offsets for unassigned partitions")
+
+        self.__validate_offsets(self.__staged_offsets)
+
         result: Optional[Sequence[ConfluentTopicPartition]]
 
         if self.__staged_offsets:
             result = self.__consumer.commit(
                 offsets=[
                     ConfluentTopicPartition(
-                        partition.topic.name, partition.index, position.offset
+                        partition.topic.name, partition.index, offset
                     )
-                    for partition, position in self.__staged_offsets.items()
+                    for partition, offset in self.__staged_offsets.items()
                 ],
                 asynchronous=False,
             )
@@ -620,7 +571,7 @@ class KafkaConsumer(Consumer[KafkaPayload]):
 
         assert result is not None  # synchronous commit should return result immediately
 
-        offsets: MutableMapping[Partition, Position] = {}
+        offsets: MutableMapping[Partition, int] = {}
 
         for value in result:
             # The Confluent Kafka Consumer will include logical offsets in the
@@ -634,15 +585,13 @@ class KafkaConsumer(Consumer[KafkaPayload]):
 
             assert value.offset >= 0, "expected non-negative offset"
             partition = Partition(Topic(value.topic), value.partition)
-            offsets[partition] = Position(
-                value.offset, self.__staged_offsets[partition].timestamp
-            )
+            offsets[partition] = value.offset
 
         self.__staged_offsets.clear()
 
         return offsets
 
-    def commit_positions(self) -> Mapping[Partition, Position]:
+    def commit_offsets(self) -> Mapping[Partition, int]:
         """
         Commit staged offsets for all partitions that this consumer is
         assigned to. The return value of this method is a mapping of
@@ -699,7 +648,7 @@ class KafkaProducer(Producer[KafkaPayload]):
 
     def __delivery_callback(
         self,
-        future: Future[Message[KafkaPayload]],
+        future: Future[BrokerValue[KafkaPayload]],
         payload: KafkaPayload,
         error: KafkaError,
         message: ConfluentMessage,
@@ -713,12 +662,12 @@ class KafkaProducer(Producer[KafkaPayload]):
                     raise ValueError("timestamp not available")
 
                 future.set_result(
-                    Message(
+                    BrokerValue(
+                        payload,
                         Partition(Topic(message.topic()), message.partition()),
                         message.offset(),
-                        payload,
                         datetime.utcfromtimestamp(timestamp_value / 1000.0),
-                    )
+                    ),
                 )
             except Exception as error:
                 future.set_exception(error)
@@ -727,7 +676,7 @@ class KafkaProducer(Producer[KafkaPayload]):
         self,
         destination: Union[Topic, Partition],
         payload: KafkaPayload,
-    ) -> Future[Message[KafkaPayload]]:
+    ) -> Future[BrokerValue[KafkaPayload]]:
         if self.__shutdown_requested.is_set():
             raise RuntimeError("producer has been closed")
 
@@ -742,7 +691,7 @@ class KafkaProducer(Producer[KafkaPayload]):
         else:
             raise TypeError("invalid destination type")
 
-        future: Future[Message[KafkaPayload]] = Future()
+        future: Future[BrokerValue[KafkaPayload]] = Future()
         future.set_running_or_notify_cancel()
         produce(
             value=payload.value,
