@@ -2,27 +2,59 @@ from __future__ import annotations
 
 import logging
 import time
-from typing import Generic, Mapping, Optional, Sequence
+from collections import defaultdict
+from enum import Enum
+from typing import Generic, Mapping, MutableMapping, Optional, Sequence
 
 from arroyo.backends.abstract import Consumer
+from arroyo.commit import CommitPolicy
 from arroyo.errors import RecoverableError
 from arroyo.processing.strategies.abstract import (
     MessageRejected,
     ProcessingStrategy,
     ProcessingStrategyFactory,
 )
-from arroyo.types import Message, Partition, Position, Topic, TPayload
+from arroyo.types import BrokerValue, Message, Partition, Topic, TPayload
 from arroyo.utils.metrics import get_metrics
 
 logger = logging.getLogger(__name__)
 
-LOG_THRESHOLD_TIME = 10  # In seconds
+METRICS_FREQUENCY_SEC = 1.0  # In seconds
 
 
 class InvalidStateError(RuntimeError):
     pass
 
 
+class ConsumerTiming(Enum):
+    CONSUMER_POLL_TIME = "arroyo.consumer.poll.time"
+    CONSUMER_PROCESSING_TIME = "arroyo.consumer.processing.time"
+    CONSUMER_PAUSED_TIME = "arroyo.consumer.paused.time"
+
+
+class MetricsBuffer:
+    def __init__(self) -> None:
+        self.__metrics = get_metrics()
+        self.__reset()
+
+    def increment(self, metric: ConsumerTiming, duration: float) -> None:
+        self.__data[metric] += duration
+        self.__throttled_record()
+
+    def flush(self) -> None:
+        for (metric, value) in self.__data.items():
+            self.__metrics.timing(metric.value, value)
+        self.__reset()
+
+    def __reset(self) -> None:
+        self.__data: MutableMapping[ConsumerTiming, float] = defaultdict(float)
+        self.__last_record_time = time.time()
+
+    def __throttled_record(self) -> None:
+        if time.time() - self.__last_record_time > METRICS_FREQUENCY_SEC:
+            self.flush()
+
+
 class StreamProcessor(Generic[TPayload]):
     """
     A stream processor manages the relationship between a ``Consumer``
@@ -36,20 +68,23 @@ class StreamProcessor(Generic[TPayload]):
         consumer: Consumer[TPayload],
         topic: Topic,
         processor_factory: ProcessingStrategyFactory[TPayload],
+        commit_policy: CommitPolicy,
+        join_timeout: Optional[float] = None,
     ) -> None:
         self.__consumer = consumer
         self.__processor_factory = processor_factory
-        self.__metrics = get_metrics()
+        self.__metrics_buffer = MetricsBuffer()
 
         self.__processing_strategy: Optional[ProcessingStrategy[TPayload]] = None
 
-        self.__message: Optional[Message[TPayload]] = None
+        self.__message: Optional[BrokerValue[TPayload]] = None
 
         # If the consumer is in the paused state, this is when the last call to
-        # ``pause`` occurred.
+        # ``pause`` occurred or the time the pause metric was last recorded.
         self.__paused_timestamp: Optional[float] = None
-        self.__last_log_timestamp: Optional[float] = None
 
+        self.__commit_policy_state = commit_policy.get_state_machine()
+        self.__join_timeout = join_timeout
         self.__shutdown_requested = False
 
         def _close_strategy() -> None:
@@ -58,13 +93,13 @@ class StreamProcessor(Generic[TPayload]):
                     "received unexpected revocation without active processing strategy"
                 )
 
-            logger.debug("Closing %r...", self.__processing_strategy)
+            logger.info("Closing %r...", self.__processing_strategy)
             self.__processing_strategy.close()
 
-            logger.debug("Waiting for %r to exit...", self.__processing_strategy)
-            self.__processing_strategy.join()
+            logger.info("Waiting for %r to exit...", self.__processing_strategy)
+            self.__processing_strategy.join(self.__join_timeout)
 
-            logger.debug(
+            logger.info(
                 "%r exited successfully, releasing assignment.",
                 self.__processing_strategy,
             )
@@ -85,6 +120,9 @@ class StreamProcessor(Generic[TPayload]):
             logger.info("New partitions assigned: %r", partitions)
             if partitions:
                 if self.__processing_strategy is not None:
+                    logger.exception(
+                        "Partition assignment while processing strategy active"
+                    )
                     _close_strategy()
                 _create_strategy(partitions)
 
@@ -112,15 +150,25 @@ class StreamProcessor(Generic[TPayload]):
             [topic], on_assign=on_partitions_assigned, on_revoke=on_partitions_revoked
         )
 
-    def __commit(self, positions: Mapping[Partition, Position]) -> None:
-        self.__consumer.stage_positions(positions)
-        start = time.time()
-        self.__consumer.commit_positions()
-        logger.debug(
-            "Waited %0.4f seconds for offsets to be committed to %r.",
-            time.time() - start,
-            self.__consumer,
-        )
+    def __commit(self, offsets: Mapping[Partition, int], force: bool = False) -> None:
+        """
+        If force is passed, commit immediately and do not throttle. This should
+        be used during consumer shutdown where we do not want to wait before committing.
+        """
+        self.__consumer.stage_offsets(offsets)
+        now = time.time()
+
+        if force or self.__commit_policy_state.should_commit(
+            now,
+            offsets,
+        ):
+            self.__consumer.commit_offsets()
+            logger.debug(
+                "Waited %0.4f seconds for offsets to be committed to %r.",
+                time.time() - now,
+                self.__consumer,
+            )
+            self.__commit_policy_state.did_commit(now, offsets)
 
     def run(self) -> None:
         "The main run loop, see class docstring for more information."
@@ -131,8 +179,8 @@ class StreamProcessor(Generic[TPayload]):
                 self._run_once()
 
             self._shutdown()
-        except Exception as error:
-            logger.info("Caught %r, shutting down...", error)
+        except Exception:
+            logger.exception("Caught exception, shutting down...")
 
             if self.__processing_strategy is not None:
                 logger.debug("Terminating %r...", self.__processing_strategy)
@@ -158,15 +206,31 @@ class StreamProcessor(Generic[TPayload]):
             # Otherwise, we need to try fetch a new message from the consumer,
             # even if there is no active assignment and/or processing strategy.
             try:
+                start_poll = time.time()
                 self.__message = self.__consumer.poll(timeout=1.0)
+                self.__metrics_buffer.increment(
+                    ConsumerTiming.CONSUMER_POLL_TIME, time.time() - start_poll
+                )
             except RecoverableError:
                 return
 
         if self.__processing_strategy is not None:
+            start_poll = time.time()
             self.__processing_strategy.poll()
+            self.__metrics_buffer.increment(
+                ConsumerTiming.CONSUMER_PROCESSING_TIME, time.time() - start_poll
+            )
             if self.__message is not None:
                 try:
-                    self.__processing_strategy.submit(self.__message)
+                    start_submit = time.time()
+                    message = (
+                        Message(self.__message) if self.__message is not None else None
+                    )
+                    self.__processing_strategy.submit(message)
+                    self.__metrics_buffer.increment(
+                        ConsumerTiming.CONSUMER_PROCESSING_TIME,
+                        time.time() - start_submit,
+                    )
                 except MessageRejected as e:
                     # If the processing strategy rejected our message, we need
                     # to pause the consumer and hold the message until it is
@@ -178,45 +242,32 @@ class StreamProcessor(Generic[TPayload]):
                             self.__message,
                         )
                         self.__consumer.pause([*self.__consumer.tell().keys()])
+
                         self.__paused_timestamp = time.time()
                     else:
-                        # Log paused condition every 5 seconds at most
                         current_time = time.time()
-                        if self.__last_log_timestamp:
-                            paused_duration: Optional[float] = (
-                                current_time - self.__last_log_timestamp
+                        if self.__paused_timestamp:
+                            self.__metrics_buffer.increment(
+                                ConsumerTiming.CONSUMER_PAUSED_TIME,
+                                current_time - self.__paused_timestamp,
                             )
-                        elif self.__paused_timestamp:
-                            paused_duration = current_time - self.__paused_timestamp
-                        else:
-                            paused_duration = None
+                            self.__paused_timestamp = current_time
 
-                        if (
-                            paused_duration is not None
-                            and paused_duration > LOG_THRESHOLD_TIME
-                        ):
-                            self.__last_log_timestamp = current_time
-                            logger.info(
-                                "Paused for longer than %d seconds", LOG_THRESHOLD_TIME
-                            )
                 else:
                     # If we were trying to submit a message that failed to be
                     # submitted on a previous run, we can resume accepting new
                     # messages.
                     if message_carried_over:
                         assert self.__paused_timestamp is not None
-                        paused_duration = time.time() - self.__paused_timestamp
-                        self.__paused_timestamp = None
-                        self.__last_log_timestamp = None
-                        logger.debug(
-                            "Successfully submitted %r, resuming consumer after %0.4f seconds...",
-                            self.__message,
-                            paused_duration,
-                        )
                         self.__consumer.resume([*self.__consumer.tell().keys()])
-                        self.__metrics.timing(
-                            "pause_duration_ms", paused_duration * 1000
+
+                        self.__metrics_buffer.increment(
+                            ConsumerTiming.CONSUMER_PAUSED_TIME,
+                            time.time() - self.__paused_timestamp,
                         )
+
+                        self.__paused_timestamp = None
+
                     self.__message = None
         else:
             if self.__message is not None:
@@ -238,6 +289,7 @@ class StreamProcessor(Generic[TPayload]):
     def _shutdown(self) -> None:
         # close the consumer
         logger.debug("Stopping consumer")
+        self.__metrics_buffer.flush()
         self.__consumer.close()
         logger.debug("Stopped")
 
