@@ -1,10 +1,16 @@
 from abc import ABC, abstractmethod
-from typing import Callable, Generic, Mapping, Optional
+from typing import Generic, Mapping, Optional
 
-from arroyo.types import Message, Partition, Position, TPayload
+from arroyo.types import Commit, Message, Partition, TPayload
 
 
 class MessageRejected(Exception):
+    """
+    MessageRejected should be raised in a processing strategy's submit method
+    if it is unable to keep up with the rate of incoming messages. It tells
+    the consumer to slow down and retry the message later.
+    """
+
     pass
 
 
@@ -101,16 +107,14 @@ class ProcessingStrategyFactory(ABC, Generic[TPayload]):
     @abstractmethod
     def create_with_partitions(
         self,
-        commit: Callable[[Mapping[Partition, Position]], None],
+        commit: Commit,
         partitions: Mapping[Partition, int],
     ) -> ProcessingStrategy[TPayload]:
         """
         Instantiate and return a ``ProcessingStrategy`` instance.
 
-        :param commit: A function that accepts a mapping of ``Partition``
-        instances to offset values that should be committed.
+        :param commit: A function that accepts a mapping of ``Partition`` instances to offset values that should be committed.
 
-        :param partitions: A mapping of a ``Partition`` to it's most recent
-        offset.
+        :param partitions: A mapping of a ``Partition`` to it's most recent offset.
         """
         raise NotImplementedError
