@@ -13,13 +13,16 @@ import pytest
 from confluent_kafka.admin import AdminClient, NewTopic
 
 from arroyo.backends.kafka import KafkaConsumer, KafkaPayload, KafkaProducer
+from arroyo.backends.kafka.commit import CommitCodec
 from arroyo.backends.kafka.configuration import build_kafka_configuration
 from arroyo.backends.kafka.consumer import as_kafka_configuration_bool
+from arroyo.commit import Commit
 from arroyo.errors import ConsumerError, EndOfPartition
-from arroyo.synchronized import Commit, commit_codec
-from arroyo.types import Message, Partition, Position, Topic
+from arroyo.types import BrokerValue, Partition, Topic
 from tests.backends.mixins import StreamsTestMixin
 
+commit_codec = CommitCodec()
+
 
 def test_payload_equality() -> None:
     assert KafkaPayload(None, b"", []) == KafkaPayload(None, b"", [])
@@ -114,9 +117,9 @@ class KafkaStreamsTestCase(StreamsTestMixin[KafkaPayload], TestCase):
             with closing(self.get_consumer(auto_offset_reset="earliest")) as consumer:
                 consumer.subscribe([topic])
 
-                message = consumer.poll(10.0)
-                assert isinstance(message, Message)
-                assert message.offset == 0
+                value = consumer.poll(10.0)
+                assert isinstance(value, BrokerValue)
+                assert value.payload.value == b"0"
 
     def test_auto_offset_reset_latest(self) -> None:
         with self.get_topic() as topic:
@@ -144,14 +147,13 @@ class KafkaStreamsTestCase(StreamsTestMixin[KafkaPayload], TestCase):
             with closing(self.get_consumer(strict_offset_reset=False)) as consumer:
                 consumer.subscribe([topic])
                 consumer.poll(10.0)  # Wait for assignment
-                consumer.stage_positions(
-                    {
-                        message.partition: Position(
-                            offset=message.offset + 1000, timestamp=message.timestamp
-                        )
-                    },
+
+                partition = Partition(topic, 0)
+
+                consumer.stage_offsets(
+                    {partition: message.offset},
                 )
-                consumer.commit_positions()
+                consumer.commit_offsets()
 
             with closing(self.get_consumer(strict_offset_reset=False)) as consumer:
                 consumer.subscribe([topic])
@@ -161,7 +163,7 @@ class KafkaStreamsTestCase(StreamsTestMixin[KafkaPayload], TestCase):
                 assert result_message.payload.value == b"0"
 
                 # make sure we reset our offset now
-                consumer.commit_positions()
+                consumer.commit_offsets()
 
     def test_auto_offset_reset_error(self) -> None:
         with self.get_topic() as topic:
@@ -175,75 +177,6 @@ class KafkaStreamsTestCase(StreamsTestMixin[KafkaPayload], TestCase):
                     consumer.poll(10.0)  # XXX: getting the subcription is slow
 
 
-def test_cooperative_rebalancing() -> None:
-    configuration = build_kafka_configuration(
-        {"bootstrap.servers": os.environ.get("DEFAULT_BROKERS", "localhost:9092")}
-    )
-
-    partitions_count = 2
-
-    group_id = uuid.uuid4().hex
-    producer = KafkaProducer(configuration)
-
-    consumer_a = KafkaConsumer(
-        {
-            **configuration,
-            "partition.assignment.strategy": "cooperative-sticky",
-            "auto.offset.reset": "earliest",
-            "enable.auto.commit": False,
-            "enable.auto.offset.store": False,
-            "group.id": group_id,
-            "session.timeout.ms": 10000,
-        },
-    )
-    consumer_b = KafkaConsumer(
-        {
-            **configuration,
-            "partition.assignment.strategy": "cooperative-sticky",
-            "auto.offset.reset": "earliest",
-            "enable.auto.commit": False,
-            "enable.auto.offset.store": False,
-            "group.id": group_id,
-            "session.timeout.ms": 10000,
-        },
-    )
-
-    with get_topic(configuration, partitions_count) as topic, closing(
-        producer
-    ), closing(consumer_a), closing(consumer_b):
-        for i in range(10):
-            for j in range(partitions_count):
-                producer.produce(
-                    Partition(topic, j),
-                    KafkaPayload(None, f"{j}-{i}".encode("utf8"), []),
-                )
-
-        consumer_a.subscribe([topic])
-
-        assert consumer_a.poll(10.0) is not None
-
-        # Consumer A has 2 partitions assigned, B has none
-        assert len(consumer_a.tell()) == 2
-        assert len(consumer_b.tell()) == 0
-
-        consumer_b.subscribe([topic])
-
-        # At some point, 1 partition will move to consumer B
-        consumer_a.pause([p for p in consumer_a.tell()])
-        for i in range(10):
-            assert consumer_a.poll(0) is None  # attempt to force session timeout
-            if consumer_b.poll(1.0) is not None:
-                break
-
-        assert len(consumer_a.tell()) == 1
-        assert len(consumer_b.tell()) == 1
-
-        # Resume A and assert that both consumer_a and consumer_b are getting messages
-        consumer_a.resume([p for p in consumer_a.tell()])
-        assert consumer_a.poll(1.0) is not None
-        assert consumer_b.poll(1.0) is not None
-
-
 def test_commit_codec() -> None:
     commit = Commit("group", Partition(Topic("topic"), 0), 0, datetime.now())
     assert commit_codec.decode(commit_codec.encode(commit)) == commit
