@@ -30,7 +30,7 @@ from arroyo.processing.strategies.dead_letter_queue.policies.produce import (
 from arroyo.processing.strategies.dead_letter_queue.policies.raise_e import (
     RaiseInvalidMessagePolicy,
 )
-from arroyo.types import Message, Partition, Topic
+from arroyo.types import BrokerValue, Message, Partition, Topic
 
 NO_KEY = "No key"
 BAD_PAYLOAD = "Bad payload"
@@ -40,14 +40,16 @@ NOW = datetime.now()
 def kafka_message_to_invalid_kafka_message(
     message: Message[KafkaPayload], reason: str
 ) -> InvalidKafkaMessage:
+    consumer_payload = message.value
+    assert isinstance(consumer_payload, BrokerValue)
     return InvalidKafkaMessage(
-        payload=message.payload.value,
-        timestamp=message.timestamp,
-        topic=message.partition.topic.name,
+        payload=consumer_payload.payload.value,
+        timestamp=consumer_payload.timestamp,
+        topic=consumer_payload.partition.topic.name,
         consumer_group="",
-        partition=message.partition.index,
-        offset=message.offset,
-        headers=message.payload.headers,
+        partition=consumer_payload.partition.index,
+        offset=consumer_payload.offset,
+        headers=consumer_payload.payload.headers,
         reason=reason,
     )
 
@@ -151,20 +153,47 @@ def processing_step() -> ProcessingStrategy[KafkaPayload]:
 
 @pytest.fixture
 def valid_message() -> Message[KafkaPayload]:
-    valid_payload = KafkaPayload(b"Key", b"Value", [])
-    return Message(Partition(Topic(""), 0), 0, valid_payload, NOW)
+    partition = Partition(Topic(""), 0)
+    offset = 0
+
+    return Message(
+        BrokerValue(
+            KafkaPayload(b"Key", b"Value", []),
+            partition,
+            offset,
+            NOW,
+        )
+    )
 
 
 @pytest.fixture
 def invalid_message_no_key() -> Message[KafkaPayload]:
-    invalid_payload = KafkaPayload(None, b"Value", [])
-    return Message(Partition(Topic(""), 0), 0, invalid_payload, NOW)
+    partition = Partition(Topic(""), 0)
+    offset = 0
+
+    return Message(
+        BrokerValue(
+            KafkaPayload(None, b"Value", []),
+            partition,
+            offset,
+            NOW,
+        )
+    )
 
 
 @pytest.fixture
 def invalid_message_bad_value() -> Message[KafkaPayload]:
-    invalid_payload = KafkaPayload(key=b"Key", value=b"\xff", headers=[])
-    return Message(Partition(Topic(""), 0), 0, invalid_payload, NOW)
+    partition = Partition(Topic(""), 0)
+    offset = 0
+
+    return Message(
+        BrokerValue(
+            KafkaPayload(key=b"Key", value=b"\xff", headers=[]),
+            partition,
+            offset,
+            NOW,
+        )
+    )
 
 
 def test_raise(
@@ -172,9 +201,7 @@ def test_raise(
     invalid_message_no_key: Message[KafkaPayload],
     processing_step: FakeProcessingStep,
 ) -> None:
-    dlq_raise: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
-        processing_step, RaiseInvalidMessagePolicy()
-    )
+    dlq_raise = DeadLetterQueue(processing_step, RaiseInvalidMessagePolicy())
     dlq_raise.submit(valid_message)
     with pytest.raises(InvalidMessages):
         dlq_raise.submit(invalid_message_no_key)
@@ -187,9 +214,7 @@ def test_ignore(
     invalid_message_no_key: Message[KafkaPayload],
     processing_step: FakeProcessingStep,
 ) -> None:
-    dlq_ignore: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
-        processing_step, IgnoreInvalidMessagePolicy()
-    )
+    dlq_ignore = DeadLetterQueue(processing_step, IgnoreInvalidMessagePolicy())
     dlq_ignore.submit(valid_message)
     dlq_ignore.submit(invalid_message_no_key)
 
@@ -197,7 +222,7 @@ def test_ignore(
 def test_dlq_join(processing_step: FakeProcessingStep) -> None:
     # processing step should raise, dlq should handle within join
     policy = IgnoreInvalidMessagePolicy()
-    dlq_ignore: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(processing_step, policy)
+    dlq_ignore = DeadLetterQueue(processing_step, policy)
     with patch.object(policy, "handle_invalid_messages") as mock:
         dlq_ignore.join()
     mock.assert_called_once()
@@ -208,7 +233,7 @@ def test_count(
     invalid_message_no_key: Message[KafkaPayload],
     processing_step: FakeProcessingStep,
 ) -> None:
-    dlq_count: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
+    dlq_count = DeadLetterQueue(
         processing_step,
         CountInvalidMessagePolicy(next_policy=IgnoreInvalidMessagePolicy(), limit=5),
     )
@@ -224,7 +249,7 @@ def test_count_short(
     invalid_message_no_key: Message[KafkaPayload],
     processing_step: FakeProcessingStep,
 ) -> None:
-    dlq_count_short: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
+    dlq_count_short = DeadLetterQueue(
         processing_step,
         CountInvalidMessagePolicy(
             next_policy=IgnoreInvalidMessagePolicy(), limit=5, seconds=1
@@ -249,7 +274,7 @@ def test_stateful_count(
     state: MutableSequence[Tuple[int, int]] = [(now - 1, 2), (now, 2)]
 
     # Stateful count DLQ intialized with 4 hits in the state
-    dlq_count_load_state: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
+    dlq_count_load_state = DeadLetterQueue(
         processing_step,
         CountInvalidMessagePolicy(
             next_policy=IgnoreInvalidMessagePolicy(),
@@ -276,9 +301,7 @@ def test_invalid_batched_messages(
     count_policy = CountInvalidMessagePolicy(
         next_policy=IgnoreInvalidMessagePolicy(), limit=5
     )
-    dlq_count: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
-        fake_batching_processor, count_policy
-    )
+    dlq_count = DeadLetterQueue(fake_batching_processor, count_policy)
 
     """
     Batch submits on 5th message, count policy raises on 6th invalid message processed.
@@ -322,9 +345,7 @@ def test_produce_invalid_messages(
     produce_policy = ProduceInvalidMessagePolicy(
         producer, Topic("test-dead-letter-topic")
     )
-    dlq_produce: DeadLetterQueue[KafkaPayload] = DeadLetterQueue(
-        processing_step, produce_policy
-    )
+    dlq_produce = DeadLetterQueue(processing_step, produce_policy)
 
     consumer = broker.get_consumer("test-group")
     consumer.subscribe([topic])
@@ -337,7 +358,9 @@ def test_produce_invalid_messages(
     dlq_produce.submit(invalid_message_no_key)
     dlq_produce.submit(invalid_message_bad_value)
 
-    produced_message = consumer.poll()
+    value = consumer.poll()
+    assert value is not None
+    produced_message = Message(value)
     assert_produced_message_is_expected(
         produced_message,
         {
@@ -353,7 +376,9 @@ def test_produce_invalid_messages(
         },
     )
 
-    produced_message = consumer.poll()
+    value = consumer.poll()
+    assert value is not None
+    produced_message = Message(value)
     assert_produced_message_is_expected(
         produced_message,
         {
@@ -371,7 +396,8 @@ def test_produce_invalid_messages(
 
 
 def assert_produced_message_is_expected(
-    produced_message: Optional[Message[KafkaPayload]], expected_dict: Mapping[str, Any]
+    produced_message: Optional[Message[KafkaPayload]],
+    expected_dict: Mapping[str, Any],
 ) -> None:
     assert produced_message is not None
     # produced message should have appropriate info
